import os
import shutil
from typing import Any
from urllib.parse import urlparse, ParseResult

import requests
from plumbum import local
from torpy.http.requests import TorRequests


def mv(url: str, path: str, replace=False, recursive=True, **kwargs):
    if os.path.exists(path) and not replace:
        return False
    shutil.move(url, path)
    return True


def https(url: str, path: str, replace=False, recursive=True, **kwargs: str):
    if os.path.exists(path) and not replace:
        return False

    qs = urlparse(url)
    res = requests.get(url, stream=True)
    with open(path, "wb") as f:
        for chunk in res.iter_content(chunk_size=kwargs.get("chunk_size", 1024)):
            if chunk:  # filter out keep-alive new chunks
                f.write(chunk)


http = https


def ssh(url: str, path: str, replace=False, recursive=True, **kwargs: str):
    if os.path.exists(path) and not replace:
        return False

    qs = urlparse(url)

    # the url[6:] is to skip the ssh://
    scp = local["scp"]
    result = scp(url[6:], path).run()
    retcode = result[0]
    return retcode == 0


def ftp(url: str, path: str, replace: bool = False, recursive=True, **kwargs: str):
    raise NotImplementedError


def sftp(url: str, path: str, replace: bool = False, recursive=True, **kwargs: str):
    raise NotImplementedError


def torrent(url: str, path: str, replace: bool = False, recursive=True, **kwargs: str):
    raise NotImplementedError


def tor(
    url: str,
    path: str,
    replace: bool = False,
    recursive: bool = True,
    proto: str = "https",
    chunk_size: int = 1024,
    **kwargs: str,
):
    qs = urlparse(url)
    scheme, netloc, _path, params, query, fragment = proto, *qs[1:]
    fixed_url = ParseResult(scheme, netloc, _path, params, query, fragment).geturl()

    with TorRequests() as tor_requests:
        with tor_requests.get_session() as sess:

            res = sess.get(fixed_url, stream=True)
            with open(path, "wb") as f:
                for chunk in res.iter_content(
                    chunk_size=kwargs.get("chunk_size", chunk_size)
                ):
                    if chunk:  # filter out keep-alive new chunks
                        f.write(chunk)


HANDLED_SUFFIXES = {
    "file": mv,
    "http": http,
    "https": https,
    "ssh": ssh,
    "ftp": ftp,
    "sftp": sftp,
    "torrent": torrent,
    "onion": tor,
    "tor": tor,
}


def _pick_downloader(url: str):
    qs = urlparse(url)
    scheme = qs.scheme
    return HANDLED_SUFFIXES.get(scheme)


def download(url: str, path: str, replace=False, recursive=True, **kwargs: Any):
    handler = _pick_downloader(url)
    if not handler:
        supported = "\n".join(HANDLED_SUFFIXES.keys())
        raise ValueError(
            f"protocol of {url} is not supported. The ones that are:\n{supported}"
        )

    return handler(url, path, replace, **kwargs)


def read(path: str, binary: bool = False):
    with open(path, mode="r" if not binary else "rb") as file:
        return file.read()



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['dwnld', 'libdwnld']

package_data = \
{'': ['*']}

install_requires = \
['plumbum>=1.7.0,<2.0.0',
 'requests>=2.26.0,<3.0.0',
 'scp>=0.13.6,<0.14.0',
 'torpy[requests]>=1.1.6,<2.0.0']

setup_kwargs = {
    'name': 'dwnld',
    'version': '0.2.1',
    'description': 'A multiprotocol downloader',
    'long_description': '# dwnld\n\n![dwnld.png](https://raw.githubusercontent.com/4thel00z/logos/master/dwnld.png)\n\n## Motivation\n\nA library to download stuff given urls with different protocols, like `ftp://...` or `ssh://...`.\n\n## Usage\n\nThere is only one interesting module level function in this repo, it can download (or move) stuff from A to B:\n\n```python\nfrom dwnld import download\n\ndownload("file://stuff.txt", "somewhere_else.txt")\ndownload("ssh://some-remote-server:/home/reptile/stuff.txt", "here.txt")\ndownload("https://cool.com/nice.pdf", "here.pdf")\ndownload("http://cool.com/nice.pdf", "here.pdf")\n# supports tor via\ndownload("onion://cool.onion/here.pdf", "here.pdf", proto="https")\n```\n\n## Todos\n\n- Support ftp/sftp\n- Support torrent\n- Support non-default ports for tor/socks4/socks5 server\n\n## License\n\nThis project is licensed under the GPL-3 license.\n',
    'author': '4thel00z',
    'author_email': '4thel00z@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/4thel00z/dwnld',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
